#ifndef R3MODELLOADER_H
#define R3MODELLOADER_H

//==============================================================================
/*! \file
 * R3Cop\n
 * Copyright (c) 2010 by Rostislav Hulik     \n
 *
 * Author:  Rostislav Hulik, ihulik@fit.vutbr.cz  \n
 * Date:    2011/2/18                          \n
 *
 * 
 * Description:
 * - Class for loading mesh data with image information
 */

#include <string>
#include <opencv/cxcore.h>
#include <R3/R3Types.h>

namespace R3
{
	/**
	 * Class loads point cloud from OpenCV xml export and saves it to OpenMesh structure
	 * It can load info about original position of point in 2D image
	 */
	class R3ModelLoader
	{
		public:
			
			/**
			 * Point type
			 */
			typedef MeshT::Point PointT;

			/**
			 * Constructor - loads a mesh from xml file
			 * If loading is done, m_loaded variable is set as true
			 * @param mesh OpenMesh structure with point cloud
			 * @param filename File with xml data
			 */
			R3ModelLoader(MeshT *mesh, std::string filename, OpenMesh::VPropHandleT<PointT> propertyCoord, OpenMesh::VPropHandleT<int> propertyIndex, int frame);

			/**
			 * Returns true, if all work with mesh is done successfully
			 */
			bool IsLoaded();
		private:

			/**
			 * Private variable - mesh is loaded?
			 */
			bool m_loaded;

			/**
			 * Private variable - mesh
			 */
			MeshT *m_mesh;
	};
} // namespace

#endif