// R3-Visualiser.cpp : Defines the entry point for the console application.
//
#include <R3OSG/OSGVisualiser.h>
#include <OpenMesh\Core\Mesh\TriMesh_ArrayKernelT.hh>


#include <osg\Geode>
#include <osgViewer\viewer>

#include <opencv\cxcore.h>
#include <opencv\cv.h>
#include <opencv\highgui.h>

#include <R3/R3ModelLoader.h>
#include <R3/R3Delaunay.h>

int main(int argc, char* argv[])
{
	using namespace OpenMesh;

	MeshT mesh;
	OpenMesh::VPropHandleT<MeshT::Point> pointHandle;
	mesh.add_property(pointHandle);

	OpenMesh::VPropHandleT<int> indexHandle;
	mesh.add_property(indexHandle);
	////////////////////////////////////////////////////////////////////////////////////
	//// calibrate camera
	//int n_boards = 10;
	//int board_n = 10;
	//CvMat* image_points = cvCreateMat(n_boards*board_n,2,CV_32FC1);
	//CvMat* object_points = cvCreateMat(n_boards*board_n,3,CV_32FC1);
	//CvMat* point_counts = cvCreateMat(n_boards,1,CV_32SC1);
	//CvMat* intrinsic_matrix = cvCreateMat(3,3,CV_32FC1);
	//CvMat* distortion_coeffs = cvCreateMat(5,1,CV_32FC1);

	//for (unsigned int i = 0; i < n_boards; ++i)
	//{
	//	MeshT aux;
	//	OpenMesh::VPropHandleT<MeshT::Point> pHandle;
	//	aux.add_property(pHandle);
	//	OpenMesh::VPropHandleT<int> iHandle;
	//	aux.add_property(iHandle);
	//	R3::R3ModelLoader loader(&aux, "output.txt", pHandle, iHandle, i * 6+1);

	//	unsigned int j = 0;
	//	for (MeshT::VertexIter vertex = aux.vertices_begin(); vertex != aux.vertices_end(); ++vertex)
	//	{
	//		if (j >= 200) break;
	//		if (j % 20 == 0)
	//		{
	//			CV_MAT_ELEM(*image_points, float,i*10 + j/20,0) = aux.property(pHandle, vertex)[0];
	//			CV_MAT_ELEM(*image_points, float,i*10 + j/20,1) = aux.property(pHandle, vertex)[1];
	//			
	//			CV_MAT_ELEM(*object_points,float,i*10 + j/20,0) = aux.point(vertex)[0];
	//			CV_MAT_ELEM(*object_points,float,i*10 + j/20,1) = aux.point(vertex)[1];
	//			CV_MAT_ELEM(*object_points,float,i*10 + j/20,2) = aux.point(vertex)[2];
	//			
	//		}

	//		
	//		++j;
	//	}
	//	
	//	

	//	std::cout << "Calibrating.... " << i+1 << " of 10" << std::endl;
	//	CV_MAT_ELEM(*point_counts, int,i,0) = 10;
	//}

	//CvCapture* capture = cvCreateFileCapture( "out.avi" );
	//	IplImage* frame;
	//	frame = cvQueryFrame( capture );
	//	frame = cvQueryFrame( capture );
	//	frame = cvQueryFrame( capture );
	//	
	//	cvInitIntrinsicParams2D(object_points, image_points, point_counts, cvGetSize( frame ), intrinsic_matrix);
	//	cvCalibrateCamera2(object_points, image_points, point_counts, cvGetSize( frame ),intrinsic_matrix, distortion_coeffs,NULL, NULL, CV_CALIB_USE_INTRINSIC_GUESS); 
	//	
	//	cvReleaseCapture( &capture );
	//std::cout << "Camera calibration - DONE" << std::endl;
	/////////////////////////////////////////////////////////////////////////////////
	R3::R3ModelLoader loader(&mesh, "output.txt", pointHandle, indexHandle, 1);
	if (!loader.IsLoaded())
	{
		std::cout << "Model failed to load..." << std::endl;
		return 1;
	}
	else
		std::cout << "Model successfully loaded..." << std::endl;
	
	R3::R3Delaunay(&mesh, pointHandle, 800, 800);
	std::cout << "Delaunay created"  << std::endl; 
	

	osg::ref_ptr<osg::Group> root = new osg::Group();
	osg::ref_ptr<osg::Geode> geode = new osg::Geode();

	osg::ref_ptr<osg::OSGVisualiser<MeshT>> geometry = new osg::OSGVisualiser<MeshT>(mesh);

	geode->addDrawable(geometry); 
	root->addChild(geode);

	// Create a viewer and show it
	osgViewer::Viewer viewer;
	viewer.setUpViewInWindow(50, 50, 800, 600);
	viewer.setSceneData( root );
	//viewer.setLightingMode(osg::View::LightingMode::NO_LIGHT);
	
	
	
	viewer.run();
	
	return 0;
}

