#include <R3/R3Delaunay.h>
#include <opencv\cxcore.h>
#include <opencv\cv.h>
#include <opencv\highgui.h>
//#include <Eigen\Array>
//#include <Eigen\LU>
namespace R3{

	R3Delaunay::R3Delaunay(MeshT * mesh, OpenMesh::VPropHandleT<PointT> propertyCoord, double height, double width)
	{
		
		m_mesh = mesh;
		m_propertyCoord = propertyCoord;
		//std::cout << m_mesh->n_vertices() << std::endl;
		MeshT::VertexHandle vertices[4];
		vertices[0] = m_mesh->add_vertex(PointT(0.0, 0.0, 0.0));
		m_mesh->property(m_propertyCoord, vertices[0]) = PointT(0.0, 0.0, 0.0);

		vertices[1] = m_mesh->add_vertex(PointT(width, 0.0, 0.0));
		m_mesh->property(m_propertyCoord, vertices[1]) = PointT(width, 0.0, 0.0);

		vertices[2] = m_mesh->add_vertex(PointT(width, height, 0.0));
		m_mesh->property(m_propertyCoord, vertices[2]) = PointT(width, height, 0.0);

		vertices[3] = m_mesh->add_vertex(PointT(0.0, height, 0.0));
		m_mesh->property(m_propertyCoord, vertices[3])
			= PointT(0.0, height, 0.0);

		m_mesh->add_face(vertices[0], vertices[1], vertices[3]);
		m_mesh->add_face(vertices[1], vertices[2], vertices[3]);

		MeshT::VertexIter end = m_mesh->vertices_end();

		
		for (MeshT::VertexIter vertex = m_mesh->vertices_begin(); vertex.handle() != vertices[0]; ++vertex)
		{
			insertIntodelaunay(vertex);
			
		}
		
		m_mesh->delete_vertex(vertices[0], false);
		m_mesh->delete_vertex(vertices[1], false);
		m_mesh->delete_vertex(vertices[2], false);
		m_mesh->delete_vertex(vertices[3], false);
		m_mesh->garbage_collection();

		m_mesh->request_face_normals();
		m_mesh->update_normals();
		for (MeshT::FaceIter face = m_mesh->faces_begin(); face != m_mesh->faces_end(); ++face)
		{
			//std::cout << "Face: " << face << std::endl;
			for (MeshT::FVIter vertex = m_mesh->fv_begin(face); vertex; ++vertex)
			{
				std::cout << "   "<<vertex.handle() << std::endl;
			}

			std::cout << m_mesh->normal(face) << std::endl;
		}
	}

	bool isCCW(R3Delaunay::PointT A, R3Delaunay::PointT B, R3Delaunay::PointT C)
	{
		R3Delaunay::PointT aux;//, A, B, C;

		/*A = a;
		B = b;
		C = c;*/

		while (! (C[0] <= A[0] && C[0] <= B[0])) 
		{
			aux = B;
			B = A;
			A = C;
			C = aux;
		}

		if (B[0] == C[0])
		{
			if (B[1] > C[1] && A[0] > B[0]) return true;
			else if (B[1] < C[1] && A[0] < B[0]) return true;
			else return false;
		}
		
		if (A[0] == C[0])
		{
			if (A[1] > C[1] && B[0] < C[0]) return true;
			else if (A[1] < C[1] && A[0] > C[0]) return true;
			else return false;
		}
		
		if ((B[1] - C[1]) / (B[0] - C[0]) > (A[1] - C[1]) / (A[0] - C[0])) return true;
				else return false;
	}

	void R3Delaunay::insertIntodelaunay(MeshT::VertexHandle vertex)
	{
		// find tri
		MeshT::FaceHandle face = findTri(vertex);
		if (face.is_valid())
		{
		std::vector<MeshT::HalfedgeHandle> toCheck;
		
		// make new edges
		MeshT::VertexHandle vertices[3];
		MeshT::FHIter halfedge = m_mesh->fh_begin(face);

		--halfedge;
		vertices[0] = m_mesh->to_vertex_handle(++halfedge);
		vertices[1] = m_mesh->to_vertex_handle(++halfedge);
		vertices[2] = m_mesh->to_vertex_handle(++halfedge);

		MeshT::Point points[3];
		MeshT::Point point;

		points[0] = m_mesh->point(vertices[0]);
		points[1] = m_mesh->point(vertices[1]);
		points[2] = m_mesh->point(vertices[2]);
		point = m_mesh->point(vertex);

		//std::cout << "deleting " << face << std::endl;
		//std::cout << "inserting " << vertex << " " << vertices[0] << " " << vertices[1] << std::endl;
		//std::cout << "inserting " << vertex << " " << vertices[1] << " " << vertices[2] << std::endl;
		//std::cout << "inserting " << vertex << " " << vertices[2] << " " << vertices[0] << std::endl;
		m_mesh->delete_face(face, false);
		//m_mesh->garbage_collection();

		//std::cout << m_mesh->n_faces() << std::endl;

		/*if (isCCW(point, points[0], points[1]))
		{
			std::cout << point << std::endl;
			std::cout << points[0] << std::endl;
			std::cout << points[1] << std::endl;
			std::cout <<"==========" << std::endl;*/
			for (MeshT::FHIter he = m_mesh->fh_begin(m_mesh->add_face(vertex, vertices[0], vertices[1])); he; ++he)
			{
				if (m_mesh->opposite_vh(he) == vertex) toCheck.push_back(he);
			}
		//}
		//else
		//{
		//	std::cout << " Flipped " << std:: endl;
		//	std::cout << point << std::endl;
		//	std::cout << points[1] << std::endl;
		//	std::cout << points[0] << std::endl;std::cout <<"==========" << std::endl;
		//	for (MeshT::FHIter he = m_mesh->fh_begin(m_mesh->add_face(vertex, vertices[1], vertices[0])); he; ++he)
		//	{	
		//		if (m_mesh->opposite_vh(he) == vertex) toCheck.push_back(he);
		//	}
		//}
		//
		////std::cout << m_mesh->n_faces() << std::endl;
		//if (isCCW(point, points[1], points[2]))
		//{
		//	std::cout << point << std::endl;
		//	std::cout << points[1] << std::endl;
		//	std::cout << points[2] << std::endl;std::cout <<"==========" << std::endl;
			for (MeshT::FHIter he = m_mesh->fh_begin(m_mesh->add_face(vertex, vertices[1], vertices[2])); he; ++he)
			{
				if (m_mesh->opposite_vh(he) == vertex) toCheck.push_back(he);
			}
		//}
		//else
		//{
		//	std::cout << "flipped"<< std::endl;
		//	std::cout << point << std::endl;
		//	std::cout << points[2] << std::endl;
		//	std::cout << points[1] << std::endl;std::cout <<"==========" << std::endl;
		//	for (MeshT::FHIter he = m_mesh->fh_begin(m_mesh->add_face(vertex, vertices[2], vertices[1])); he; ++he)
		//	{
		//		if (m_mesh->opposite_vh(he) == vertex) toCheck.push_back(he);
		//	}
		//}

		////std::cout << m_mesh->n_faces() << std::endl;
		//if (isCCW(point, points[2], points[0]))
		//{
		//	std::cout << point << std::endl;
		//	std::cout << points[2] << std::endl;
		//	std::cout << points[0] << std::endl;std::cout <<"==========" << std::endl;
			for (MeshT::FHIter he = m_mesh->fh_begin(m_mesh->add_face(vertex, vertices[2], vertices[0])); he; ++he)
			{
				if (m_mesh->opposite_vh(he) == vertex) toCheck.push_back(he);
			}
		/*}
		else
		{
			std::cout << "flipped"<< std::endl;
			std::cout << point << std::endl;
			std::cout << points[0] << std::endl;
			std::cout << points[2] << std::endl;std::cout <<"==========" << std::endl;
			for (MeshT::FHIter he = m_mesh->fh_begin(m_mesh->add_face(vertex, vertices[0], vertices[2])); he; ++he)
			{
				if (m_mesh->opposite_vh(he) == vertex) toCheck.push_back(he);
			}
		}*/
		
		
		// check and flip
		while (!toCheck.empty())
		{
			MeshT::HalfedgeHandle hedge = toCheck.back();
			toCheck.pop_back();

			// if point is in circumcentre, flip and check neighbours
			//std::cout << m_mesh->to_vertex_handle(hedge)<< "-" << m_mesh->is_boundary(hedge) << " --> ";
			//std::cout << m_mesh->to_vertex_handle(m_mesh->opposite_halfedge_handle(hedge)) << "-" << m_mesh->is_boundary(m_mesh->opposite_halfedge_handle(hedge)) << std::endl;;
			if (!m_mesh->is_boundary(hedge) && !m_mesh->is_boundary(m_mesh->opposite_halfedge_handle(hedge)) && isInCircumCircle(hedge) && m_mesh->is_flip_ok(m_mesh->edge_handle(hedge)))
			{
				
				m_mesh->flip(m_mesh->edge_handle(hedge));

				
				// find two to check
				if (m_mesh->to_vertex_handle(hedge) != vertex) hedge = m_mesh->opposite_halfedge_handle(hedge);

				//push hedge->hedge->hedge
				toCheck.push_back(m_mesh->next_halfedge_handle(m_mesh->next_halfedge_handle(hedge)));
				//push opp(hedge)->hedge
				toCheck.push_back(m_mesh->next_halfedge_handle(m_mesh->opposite_halfedge_handle(hedge)));
			}
		}

		//for (MeshT::FaceIter face = m_mesh->faces_begin(); face != m_mesh->faces_end(); ++face)
		//{
		//	if (!m_mesh->status(face).deleted())
		//	{
		//	//std::cout << "Face: " << face << std::endl;
		//	for (MeshT::FVIter vertex = m_mesh->fv_begin(face); vertex; ++vertex)
		//	{
		//		std::cout << "   "<<vertex.handle() << std::endl;
		//	}

		//	std::cout << std::endl;
		//	}
		//	else
		//	{
		//		std::cout << "Face: " << face << " deleted." << std::endl;
		//	}
		//}
		}
	}

	bool triSameSide(R3Delaunay::PointT p1, R3Delaunay::PointT p2, R3Delaunay::PointT a ,R3Delaunay::PointT b)
	{
		R3Delaunay::PointT cp1 = (b-a) % (p1-a);
		R3Delaunay::PointT cp2 = (b-a) % (p2-a);
    
		if ((cp1 | cp2) >= 0) return true;
		else return false;
	}

	bool pointInTriangle(R3Delaunay::PointT p, R3Delaunay::PointT a, R3Delaunay::PointT b, R3Delaunay::PointT c)
	{
		/*if (triSameSide(p,a, b,c) && triSameSide(p,b, a,c) && triSameSide(p,c, a,b)) return true;
		else return false;*/
		//std::cout << a << std::endl;
		//std::cout << b << std::endl;
		//std::cout << c << std::endl;
		//std::cout << p << std::endl;
		//std::cout << "-----" << std::endl;

		MeshT::Normal v0 = c - a;
		MeshT::Normal v1 = b - a;
		MeshT::Normal v2 = p - a;

		double dot00 = dot(v0, v0);
		double dot01 = dot(v0, v1);
		double dot02 = dot(v0, v2);
		double dot11 = dot(v1, v1);
		double dot12 = dot(v1, v2);

		// Compute barycentric coordinates
		double invDenom = 1.0 / (dot00 * dot11 - dot01 * dot01);
		double u = (dot11 * dot02 - dot01 * dot12) * invDenom;
		double v = (dot00 * dot12 - dot01 * dot02) * invDenom;

		// Check if point is in triangle
		return (u > 0) && (v > 0) && (u + v < 1);
	}

	

	MeshT::FaceHandle R3Delaunay::findTri(MeshT::VertexHandle vertex)
	{
		MeshT::FaceIter end = m_mesh->faces_end();
		for (MeshT::FaceIter face = m_mesh->faces_begin(); face !=end; ++face)
		{
			MeshT::FVIter vertexAround = m_mesh->fv_begin(face);
			/*MeshT::FVIter vertexAround2, vertexAround3;
			vertexAround2 = vertexAround;
			vertexAround3 = vertexAround;

			++vertexAround2;
			++vertexAround3;
			++vertexAround3;*/
			--vertexAround;
			if (!m_mesh->status(face).deleted() && 
				pointInTriangle(	m_mesh->property(m_propertyCoord, vertex),
									m_mesh->property(m_propertyCoord, ++vertexAround),
									m_mesh->property(m_propertyCoord, ++vertexAround),
									m_mesh->property(m_propertyCoord, ++vertexAround)))
			{
				return face;
			}
		}

		return MeshT::InvalidFaceHandle;
	}

	bool R3Delaunay::isInCircumCircle(MeshT::HalfedgeHandle hedge)
	{
		MeshT::FHIter hedgeIter = m_mesh->fh_begin(m_mesh->face_handle(hedge));
		--hedgeIter;
		MeshT::Point A = m_mesh->property(m_propertyCoord, m_mesh->to_vertex_handle(++hedgeIter));
		MeshT::Point B = m_mesh->property(m_propertyCoord, m_mesh->to_vertex_handle(++hedgeIter));
		MeshT::Point C = m_mesh->property(m_propertyCoord, m_mesh->to_vertex_handle(++hedgeIter));

		/*if ((B[1] - C[1]) / (B[0] - C[0]) <= (A[1] - C[1]) / (A[0] - C[0]))
		{
			MeshT::Point aux = A;
			A = C;
			C = aux;
			

		}*/
		/*std::cout << A << std::endl;
		std::cout << B << std::endl;
		std::cout << C << std::endl;*/
		hedgeIter = m_mesh->fh_begin(m_mesh->face_handle(m_mesh->opposite_halfedge_handle(hedge)));
		++hedgeIter;
		MeshT::Point D = m_mesh->property(m_propertyCoord, m_mesh->to_vertex_handle(hedgeIter));
		/////////////////////////////////////////////////////////////////////////////
		CvMat* matrix = cvCreateMat(3, 3, CV_32FC1);
		
			
		cvmSet(matrix, 0, 0, A[0] - D[0]);	cvmSet(matrix, 0,1, A[1] - D[1]);	cvmSet(matrix, 0,2, (A[0]*A[0] - D[0]*D[0]) + (A[1]*A[1] - D[1]*D[1]));
		cvmSet(matrix, 1, 0, B[0] - D[0]);	cvmSet(matrix, 1,1, B[1] - D[1]);	cvmSet(matrix, 1,2, (B[0]*B[0] - D[0]*D[0]) + (B[1]*B[1] - D[1]*D[1]));
		cvmSet(matrix, 2, 0, C[0] - D[0]);	cvmSet(matrix, 2,1, C[1] - D[1]);	cvmSet(matrix, 2,2, (C[0]*C[0] - D[0]*D[0]) + (C[1]*C[1] - D[1]*D[1]));
		
		return (cvDet(matrix) > 0);
		//////////////////////////////////////////////////////////////////////////////////
		//std::cout << matrix.determinant() << std::endl;
		
		
	
		/*MeshT::FHIter hedgeIter = m_mesh->fh_begin(m_mesh->face_handle(hedge));
		--hedgeIter;
		MeshT::Point point1 = m_mesh->property(m_propertyCoord, m_mesh->to_vertex_handle(++hedgeIter));
		MeshT::Point point2 = m_mesh->property(m_propertyCoord, m_mesh->to_vertex_handle(++hedgeIter));
		MeshT::Point point3 = m_mesh->property(m_propertyCoord, m_mesh->to_vertex_handle(++hedgeIter));

		MeshT::Normal vector1 = point2 - point1;
		vector1.normalize_cond();
		vector1[2] = 0.0;
		MeshT::Normal vector2 = point3 - point2;
		vector2.normalize_cond();
		vector2[2] = 0.0;

		double angle = acos(vector1 | vector2);
		
		hedgeIter = m_mesh->fh_begin(m_mesh->face_handle(m_mesh->opposite_halfedge_handle(hedge)));
		--hedgeIter;
		point1 = m_mesh->property(m_propertyCoord, m_mesh->to_vertex_handle(++hedgeIter));
		point2 = m_mesh->property(m_propertyCoord, m_mesh->to_vertex_handle(++hedgeIter));
		point3 = m_mesh->property(m_propertyCoord, m_mesh->to_vertex_handle(++hedgeIter));

		vector1 = point2 - point1;
		vector1.normalize_cond();
		vector1[2] = 0.0;
		vector2 = point3 - point2;
		vector2.normalize_cond();
		vector2[2] = 0.0;

		angle += acos(vector1 | vector2);

		return (angle > M_PI);*/
	}


	
	
}