//==============================================================================
/*! \file
 * R3Cop\n
 * Copyright (c) 2010 by Rostislav Hulik     \n
 *
 * Author:  Rostislav Hulik, ihulik@fit.vutbr.cz  \n
 * Date:    2011/2/18                          \n
 *
 * 
 * Description:
 * - Class for loading mesh data with image information
 */

#include <R3/R3ModelLoader.h>
#include <fstream>
namespace R3 {
	
////////////////////////////////////////////////////////////////////////////////////////////
// Constructor - loads a mesh from xml file
// If loading is done, m_loaded variable is set as true
// @param mesh OpenMesh structure with point cloud
// @param filename File with xml data
////////////////////////////////////////////////////////////////////////////////////////////
R3ModelLoader::R3ModelLoader(MeshT *mesh, std::string filename, OpenMesh::VPropHandleT<PointT> propertyCoord, OpenMesh::VPropHandleT<int> propertyIndex, int frame)
{
	m_loaded = false;
	m_mesh = mesh;

	std::ifstream file;
	file.open(filename);

	if (!file.is_open() || !propertyCoord.is_valid() || !propertyCoord.is_valid()) return;

	MeshT::Scalar pts[3];
	MeshT::Scalar pic[3];
	int index, indexFrame;

	int num;

	file >> indexFrame;
	file >> num;

	while (!file.eof())
	{
		
		
		for (int i = 0; i < num; ++i)
			{
				file >> index;
				file >> pts[0];
				file >> pts[1];
				file >> pts[2];

				file >> pic[0];
				file >> pic[1];
				pic[2] = 0.0;
				if (frame == indexFrame)
				{
				MeshT::VertexHandle handle = mesh->add_vertex( PointT(pts[0], pts[1], pts[2]) );
				mesh->property(propertyCoord, handle) = PointT(pic[0], pic[1], pic[2]);
				mesh->property(propertyIndex, handle) = index;
				}
			}
		
		
		file >> indexFrame;
		file >> num;

		
	}

	m_loaded = true;
}

//////////////////////////////////////////////////////////////////////////////////////////////
//// Function loads the xml file with information connecting points in point cloud and original 2D image
//// Also sets m_loaded as tru, if loading is done successfully
//// @param filename File with xml data
//// @param propertyHandle Mesh property with coordinates in original image
//// @return true, if loading is done without error
//////////////////////////////////////////////////////////////////////////////////////////////
//bool R3ModelLoader::LoadImageCoords(std::string filename, OpenMesh::VPropHandleT<PointT> propertyHandle)
//{
//	CvMat *points = (CvMat*)cvLoad(filename.c_str()); 
//
//	if (points == NULL || points->rows != 1 || m_mesh->n_vertices() != points->cols || !m_loaded) 
//	{
//		m_loaded = false;
//		return false;
//	}
//
//	MeshT::VertexIter vertex = m_mesh->vertices_begin();
//	for (int i = 0; i < points->cols; i++, ++vertex)
//	{
//		m_mesh->property(propertyHandle, vertex) = PointT(	(float) cvGet2D(points, 0, i).val[0], 
//															(float)-cvGet2D(points, 0, i).val[1], 
//															0.0f);
//	}
//	return true;
//}

////////////////////////////////////////////////////////////////////////////////////////////
// Returns true, if all work with mesh is done successfully
////////////////////////////////////////////////////////////////////////////////////////////
bool R3ModelLoader::IsLoaded()
{
	return m_loaded;
}
}